@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # 
-# Copyright (c) 2009-2013 Felix Krull <f_krull@gmx.de>
+# Copyright (c) 2009-2014 Felix Krull <f_krull@gmx.de>
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ documentation or use the ``calculate`` function.
 """
 
 import os.path
+import sys
 
 import gi
 gi.require_version('Gst', '1.0')
@@ -28,14 +29,17 @@ from gi.repository import GObject, Gst
 
 from rgain import GainData, GSTError, util
 
-# Make sure GObject threads don't crash
+# Initialise threading.
 GObject.threads_init()
-
-def to_utf8(string):
-    if isinstance(string, unicode):
-        return string.encode("utf-8")
-    else:
-        return string.decode("utf-8").encode("utf-8")
+# Also initialise threading. This hack is necessary because while threads_init
+# was originally deprecated in pygobject 3.10 and turned into a no-op, that
+# wouldn't initialise Python threading properly when an introspection-loaded
+# library used threading, but the Python program didn't (see
+# https://bugzilla.gnome.org/show_bug.cgi?id=710447). Therefore, we create a
+# dummy thread to force Python to initialise threading to accomodate these
+# broken pygobject versions.
+import threading
+threading.Thread(target=lambda: None).start()
 
 class ReplayGain(GObject.GObject):
     
@@ -109,22 +113,34 @@ class ReplayGain(GObject.GObject):
     
     
     # internal stuff
+    def _check_elem(self, elem):
+        if elem is None:
+            # that element couldn't be created, maybe because plugins are
+            # missing?
+            raise Exception(u"failed to construct pipeline (did you install "
+                            u"all necessary GStreamer plugins?)")
+        else:
+            return elem
     def _setup_pipeline(self):
         """Setup the pipeline."""
         self.pipe = Gst.Pipeline()
         
         # elements
-        self.src = Gst.ElementFactory.make("filesrc", "src")
+        self.src = self._check_elem(Gst.ElementFactory.make("filesrc", "src"))
         self.pipe.add(self.src)
-        self.decbin = Gst.ElementFactory.make("decodebin", "decbin")
+        self.decbin = self._check_elem(Gst.ElementFactory.make("decodebin",
+                                                               "decbin"))
         self.pipe.add(self.decbin)
-        self.conv = Gst.ElementFactory.make("audioconvert", "conv")
+        self.conv = self._check_elem(Gst.ElementFactory.make("audioconvert",
+                                                             "conv"))
         self.pipe.add(self.conv)
-        self.res = Gst.ElementFactory.make("audioresample", "res")
+        self.res = self._check_elem(Gst.ElementFactory.make("audioresample",
+                                                            "res"))
         self.pipe.add(self.res)
-        self.rg = Gst.ElementFactory.make("rganalysis", "rg")
+        self.rg = self._check_elem(Gst.ElementFactory.make("rganalysis", "rg"))
         self.pipe.add(self.rg)
-        self.sink = Gst.ElementFactory.make("fakesink", "sink")
+        self.sink = self._check_elem(Gst.ElementFactory.make("fakesink",
+                                                             "sink"))
         self.pipe.add(self.sink)
         
         # Set num-tracks to the number of files we have to process so they're
@@ -160,11 +176,21 @@ class ReplayGain(GObject.GObject):
         except StopIteration:
             self.emit("all-finished", self.track_data, self.album_data)
             return False
-        
-        # point the source to the new file
-        self.src.set_property("location", to_utf8(fname))
+
+        # By default, GLib (and therefore GStreamer) assume any filename to be
+        # UTF-8 encoded, regardless of locale settings (though most Unix
+        # systems, Linux at least, should be configured for UTF-8 anyways these
+        # days). The file name we pass to GStreamer is encoded with the system
+        # default encoding here: if that's UTF-8, everyone's happy, if it isn't,
+        # GLib's UTF-8 assumption needs to be overridden using the
+        # G_FILENAME_ENCODING environment variable (set to locale to tell GLib
+        # that all file names passed to it are encoded in the system encoding).
+        # That way, people on non-UTF-8 systems or with non-UTF-8 file names can
+        # still force all file name processing into a different encoding.
+        self.src.set_property("location",
+                              fname.encode(util.getfilesystemencoding()))
         self._current_file = fname
-        self.emit("track-started", to_utf8(fname))
+        self.emit("track-started", fname)
         
         return True
     
@@ -204,7 +230,7 @@ class ReplayGain(GObject.GObject):
         if msg.type == Gst.MessageType.TAG:
             self._process_tags(msg)
         elif msg.type == Gst.MessageType.EOS:
-            self.emit("track-finished", to_utf8(self._current_file),
+            self.emit("track-finished", self._current_file,
                       self.track_data[self._current_file])
             # Preserve rganalysis state
             self.rg.set_locked_state(True)
